/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BenutzerE2EComponent, BenutzerListE2EComponent, BenutzerListItemE2EComponent, } from 'apps/admin-e2e/src/components/benutzer/benutzer.e2e.component';
import { E2EBenutzerHelper } from 'apps/admin-e2e/src/helper/benutzer/benutzer.helper';
import { OrganisationsEinheitE2E } from 'apps/admin-e2e/src/model/organisations-einheit';
import { beChecked, beEnabled, contains, exist, mouseEnter, notBeChecked, notBeEnabled, visible, } from 'apps/admin-e2e/src/support/cypress.util';
import { AlfaRollen, AlfaUsers, loginAsAriane } from 'apps/admin-e2e/src/support/user-util';

describe('Benutzer und Rollen', () => {
  const benutzerPage: BenutzerE2EComponent = new BenutzerE2EComponent();
  const benutzerListPage: BenutzerListE2EComponent = new BenutzerListE2EComponent();

  //TODO In der entsprechenden json hinterlegen und darauf zugreifen
  const emailAddress: string = 'peter.von.der.post@ozg-sh.de';

  const helper: E2EBenutzerHelper = new E2EBenutzerHelper();

  before(() => {
    loginAsAriane();
  });

  it('should show users and attributes in list', () => {
    helper.openBenutzerListPage();

    const ariane: BenutzerListItemE2EComponent = benutzerListPage.getItem(AlfaUsers.ARIANE);
    exist(ariane.getRoot());
    contains(ariane.getRoles(), AlfaRollen.USER);

    const dorothea: BenutzerListItemE2EComponent = benutzerListPage.getItem(AlfaUsers.DOROTHEA);
    exist(dorothea.getRoot());
    contains(dorothea.getRoles(), AlfaRollen.USER);

    const ludwig: BenutzerListItemE2EComponent = benutzerListPage.getItem(AlfaUsers.LUDWIG);
    exist(ludwig.getRoot());
    contains(ludwig.getRoles(), AlfaRollen.LOESCHEN);
    contains(ludwig.getOrganisationsEinheiten(), OrganisationsEinheitE2E.ORDNUNGSAMT);

    const zelda: BenutzerListItemE2EComponent = benutzerListPage.getItem(AlfaUsers.ZELDA);
    exist(zelda.getRoot());
    contains(zelda.getRoles(), AlfaRollen.USER);
    contains(zelda.getOrganisationsEinheiten(), OrganisationsEinheitE2E.DENKMALPFLEGE);
    contains(zelda.getOrganisationsEinheiten(), OrganisationsEinheitE2E.WIRTSCHAFTSFOERDERUNG);

    const peter: BenutzerListItemE2EComponent = benutzerListPage.getItem(AlfaUsers.PETER);
    exist(peter.getRoot());
    contains(peter.getRoles(), AlfaRollen.POSTSTELLE);
    contains(peter.getEMail(), emailAddress);

    const richard: BenutzerListItemE2EComponent = benutzerListPage.getItem(AlfaUsers.RICHARD);
    exist(richard.getRoot());
    exist(richard.getNoOrganisationsEinheitText());
  });

  it('should show checkbox for each role', () => {
    helper.openNewBenutzerPage();

    notBeChecked(benutzerPage.getAdminCheckbox().getRoot());
    notBeChecked(benutzerPage.getDatenbeauftragungCheckbox().getRoot());
    notBeChecked(benutzerPage.getLoeschenCheckbox().getRoot());
    notBeChecked(benutzerPage.getUserCheckbox().getRoot());
    notBeChecked(benutzerPage.getPostCheckbox().getRoot());
  });

  it('should deactivate other alfa roles if "loeschen" role is selected', () => {
    benutzerPage.getLoeschenCheckbox().getRoot().click();
    beChecked(benutzerPage.getLoeschenCheckbox().getRoot());
    notBeEnabled(benutzerPage.getUserCheckbox().getRoot());
    notBeEnabled(benutzerPage.getPostCheckbox().getRoot());

    benutzerPage.getLoeschenCheckbox().getRoot().click();
    notBeChecked(benutzerPage.getLoeschenCheckbox().getRoot());
    beEnabled(benutzerPage.getUserCheckbox().getRoot());
    beEnabled(benutzerPage.getPostCheckbox().getRoot());
  });

  it('should deactivate other alfa roles if "user" role is selected', () => {
    benutzerPage.getUserCheckbox().getRoot().click();
    beChecked(benutzerPage.getUserCheckbox().getRoot());
    notBeEnabled(benutzerPage.getLoeschenCheckbox().getRoot());
    notBeEnabled(benutzerPage.getPostCheckbox().getRoot());

    benutzerPage.getUserCheckbox().getRoot().click();
    notBeChecked(benutzerPage.getUserCheckbox().getRoot());
    beEnabled(benutzerPage.getLoeschenCheckbox().getRoot());
    beEnabled(benutzerPage.getPostCheckbox().getRoot());
  });

  it('should deactivate other alfa roles if "poststelle" role is selected', () => {
    benutzerPage.getPostCheckbox().getRoot().click();
    beChecked(benutzerPage.getPostCheckbox().getRoot());
    notBeEnabled(benutzerPage.getLoeschenCheckbox().getRoot());
    notBeEnabled(benutzerPage.getUserCheckbox().getRoot());

    benutzerPage.getPostCheckbox().getRoot().click();
    notBeChecked(benutzerPage.getPostCheckbox().getRoot());
    beEnabled(benutzerPage.getLoeschenCheckbox().getRoot());
    beEnabled(benutzerPage.getUserCheckbox().getRoot());
  });

  it('should activate and deactivate admin roles', () => {
    benutzerPage.getAdminCheckbox().getRoot().click();
    benutzerPage.getDatenbeauftragungCheckbox().getRoot().click();
    beChecked(benutzerPage.getAdminCheckbox().getRoot());
    beChecked(benutzerPage.getDatenbeauftragungCheckbox().getRoot());

    benutzerPage.getAdminCheckbox().getRoot().click();
    benutzerPage.getDatenbeauftragungCheckbox().getRoot().click();
    notBeChecked(benutzerPage.getAdminCheckbox().getRoot());
    notBeChecked(benutzerPage.getDatenbeauftragungCheckbox().getRoot());
  });

  describe('hint text', () => {
    it('should be visible on admin role mouse hover', () => {
      mouseEnter(benutzerPage.getAdminCheckbox().getInfoButton());

      visible(benutzerPage.getAdminCheckbox().getInfoButtonTooltip());
    });

    it('should be visible on loeschen role mouse hover', () => {
      mouseEnter(benutzerPage.getLoeschenCheckbox().getInfoButton());

      visible(benutzerPage.getLoeschenCheckbox().getInfoButtonTooltip());
    });

    it('should be visible on user role mouse hover', () => {
      mouseEnter(benutzerPage.getUserCheckbox().getInfoButton());

      visible(benutzerPage.getUserCheckbox().getInfoButtonTooltip());
    });

    it('should be visible on poststellt role mouse hover', () => {
      mouseEnter(benutzerPage.getPostCheckbox().getInfoButton());

      visible(benutzerPage.getPostCheckbox().getInfoButtonTooltip());
    });

    it('should be visible on datenbeauftragung role mouse hover', () => {
      mouseEnter(benutzerPage.getDatenbeauftragungCheckbox().getInfoButton());

      visible(benutzerPage.getDatenbeauftragungCheckbox().getInfoButtonTooltip());
    });
  });
});
