import {
  BenutzerDeleteDialogE2EComponent,
  BenutzerE2EComponent,
  BenutzerListE2EComponent,
} from '../../components/benutzer/benutzer.e2e.component';
import { SnackBarE2EComponent } from '../../components/ui/snackbar.e2e.component';
import { OrganisationsEinheitE2E } from '../../model/organisations-einheit';
import { AdminUserE2E } from '../../model/util';
import { exist, notExist } from '../../support/cypress.util';

export class E2EBenutzerExecutor {
  private benutzerPage: BenutzerE2EComponent = new BenutzerE2EComponent();
  private snackBar: SnackBarE2EComponent = new SnackBarE2EComponent();
  private benutzerDeleteDialog: BenutzerDeleteDialogE2EComponent = new BenutzerDeleteDialogE2EComponent();
  private benutzerListPage: BenutzerListE2EComponent = new BenutzerListE2EComponent();

  public modifyBenutzer(user: AdminUserE2E): void {
    this.benutzerPage.getVornameInput().type(user.vorname);
    this.benutzerPage.getNachnameInput().type(user.nachname);
    this.benutzerPage.getBenutzernameInput().type(user.username);
    this.benutzerPage.getMailInput().type(user.email);

    if (user.isAdmin) {
      this.benutzerPage.getAdminCheckbox().getRoot().click();
    }
    if (user.isUser) {
      this.benutzerPage.getUserCheckbox().getRoot().click();
    }
    if (user.isLoeschen) {
      this.benutzerPage.getLoeschenCheckbox().getRoot().click();
    }
    if (user.isPoststelle) {
      this.benutzerPage.getPostCheckbox().getRoot().click();
    }

    this.modifyOrganisationsEinheiten(user.organisationseinheiten);
  }

  public modifyOrganisationsEinheiten(organisationseinheiten: OrganisationsEinheitE2E[]): void {
    organisationseinheiten.forEach((name: string) => this.benutzerPage.getOrganisationsEinheitCheckbox(name).click());
  }

  public saveAndCloseSnackbar(): void {
    this.saveBenutzer();
    exist(this.snackBar.getMessage());
    this.snackBar.getCloseButton().click();
    notExist(this.snackBar.getMessage());
    exist(this.benutzerListPage.getList());
  }

  public saveBenutzer(): void {
    this.benutzerPage.getSaveButton().click();
  }

  public deleteBenutzer(): void {
    this.benutzerPage.getDeleteButton().click();
    exist(this.benutzerDeleteDialog.getDeleteButton());
    this.benutzerDeleteDialog.getDeleteButton().click();
    exist(this.benutzerListPage.getList());
  }
}
