/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { UserE2E } from '../model/user';
import { HeaderE2EComponent } from '../page-objects/header.po';
import { MainPage, waitForSpinnerToDisappear } from '../page-objects/main.po';
import { wait } from './cypress-helper';

export function loginAsAriane(): void {
  login(UserJsonPath.ARIANE);
}

export function loginAsDaria(): void {
  login(UserJsonPath.DARIA);
}

export function loginAsSabine(): void {
  login(UserJsonPath.SABINE);
}

export function loginAsSafira(): void {
  login(UserJsonPath.SAFIRA);
}

function login(userJson: string): void {
  cy.fixture(userJson).then((user) => loginByUi(user));
  waitForSpinnerToDisappear();
  wait(3000, 'Wait for initial navigation is done');
}

// Hinweis: cacheAcrossSpecs: true lässt Tests umfallen
export function loginByUi(user: UserE2E): void {
  cy.session(
    user.name,
    () => {
      cy.visit('/')
        .get('#kc-login')
        .should('exist')
        .then(() => {
          cy.get('#username').type(user.name);
          cy.get('#password').type(user.password);
          cy.get('#kc-login').click();
        });
    },
    {
      cacheAcrossSpecs: false,
      validate() {
        cy.window().its('sessionStorage').invoke('getItem', 'access_token').should('not.be.empty');
      },
    },
  );
}

enum UserJsonPath {
  ARIANE = 'user/user_ariane.json',
  DARIA = 'user/user_daria.json',
  SABINE = 'user/user_sabine.json',
  SAFIRA = 'user/user_safira.json',
}

export function logout(): void {
  const mainPage: MainPage = new MainPage();
  const header: HeaderE2EComponent = mainPage.getHeader();
  header.getCurrentUserProfile().logout();
}

export enum AlfaRollen {
  USER = 'VERWALTUNG_USER',
  LOESCHEN = 'VERWALTUNG_LOESCHEN',
  POSTSTELLE = 'VERWALTUNG_POSTSTELLE',
  ADMIN = 'ADMIN_ADMIN',
  DATEN = 'DATENBEAUFTRAGUNG',
}

export enum AlfaUsers {
  ARIANE = 'ariane',
  DARIA = 'daria',
  DOROTHEA = 'dorothea',
  LUDWIG = 'ludwig',
  PETER = 'peter',
  RICHARD = 'richard',
  ZELDA = 'zelda',
}
