import { dispatchEventFromFixture, existsAsHtmlElement, mock, Mock, MockEvent } from '@alfa-client/test-utils';
import { OzgcloudDialogService } from '@alfa-client/ui';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ButtonComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { CancelDialogButtonComponent } from './cancel-dialog-button.component';

describe('CancelDialogButtonComponent', () => {
  let component: CancelDialogButtonComponent;
  let fixture: ComponentFixture<CancelDialogButtonComponent>;

  let dialogService: Mock<OzgcloudDialogService>;

  const cancelDialogButton: string = getDataTestIdOf('cancel-dialog-button');
  const cancelDialogIconButton: string = getDataTestIdOf('cancel-dialog-icon-button');

  beforeEach(async () => {
    dialogService = mock(OzgcloudDialogService);

    await TestBed.configureTestingModule({
      imports: [CancelDialogButtonComponent],
      declarations: [MockComponent(ButtonComponent)],
      providers: [
        {
          provide: OzgcloudDialogService,
          useValue: dialogService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(CancelDialogButtonComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('on button click', () => {
    it('should call dialog service to close all', () => {
      dispatchEventFromFixture(fixture, cancelDialogButton, MockEvent.CLICK);

      expect(dialogService.closeAll).toHaveBeenCalled();
    });
  });

  describe('template', () => {
    describe('button', () => {
      beforeEach(() => {
        component.showAsIconButton = false;

        fixture.detectChanges();
      });

      it('should be visible when showAsIconButton is false', () => {
        existsAsHtmlElement(fixture, cancelDialogButton);
      });

      it('should call dialog service to close all', () => {
        dispatchEventFromFixture(fixture, cancelDialogButton, MockEvent.CLICK);

        expect(dialogService.closeAll).toHaveBeenCalled();
      });
    });

    describe('icon button', () => {
      beforeEach(() => {
        component.showAsIconButton = true;

        fixture.detectChanges();
      });

      it('should be visible when showAsIconButton is true', () => {
        component.showAsIconButton = true;

        fixture.detectChanges();

        existsAsHtmlElement(fixture, cancelDialogIconButton);
      });

      it('should call dialog service to close all', () => {
        dispatchEventFromFixture(fixture, cancelDialogIconButton, MockEvent.CLICK);

        expect(dialogService.closeAll).toHaveBeenCalled();
      });
    });
  });
});
